@@ -67,7 +67,7 @@ void draw_desktop(void)
     move(0, 0);
     printw(sp_buf, ' ');	/* these two print the top n' bottom */
     move(LINES - 1, 0);
-    printw(sp_buf, 'a');	/* lines */
+    printw(sp_buf, ' ');	/* lines */
 
     attrset(FIELDATTR);
 
@@ -165,6 +165,19 @@ void about()
     doupdate();
 }
 
+void show_sort_statwin(WINDOW **statwin, PANEL **panel)
+{
+    *statwin = newwin(5, 30, (LINES - 5) / 2, (COLS - 30) / 2);
+    *panel = new_panel(*statwin);
+    
+    wattrset(*statwin, BOXATTR);
+    colorwin(*statwin);
+    box(*statwin, ACS_VLINE, ACS_HLINE);
+    
+    wattrset(*statwin, STDATTR);
+    mvwprintw(*statwin, 2, 2, "Sorting, please wait...");
+}
+
 void printnomem()
 {
     attrset(ERRTXTATTR);
@@ -239,21 +252,6 @@ void indicate(char *message)
     refresh();
 }
 
-void show_sort_statwin(WINDOW **statwin, PANEL **panel)
-{
-    *statwin = newwin(5, 20, (LINES - 5) / 2, (COLS - 20) / 2);
-    *panel = new_panel(*statwin);
-    
-    wattrset(*statwin, BOXATTR);
-    colorwin(*statwin);
-    box(*statwin, ACS_VLINE, ACS_HLINE);
-    
-    mvwprintw(*statwin, 0, 1, " Sorting ");
-    wattrset(*statwin, STDATTR);
-    mvwprintw(*statwin, 2, 2, "Completed:");
-    wattrset(*statwin, HIGHATTR);
-}
-
 void printlargenum(unsigned long long i, WINDOW * win)
 {
     if (i < 100000000)		/* less than 100 million */
