@@ -6,9 +6,8 @@ others
 
 ***/
 
-void ipmon(const struct OPTIONS *options, int filtered, struct filterlist *fl,
-	   struct othpoptions *ofilter, unsigned int tm, int facilitytime,
-	   char *ifptr);
+void ipmon(const struct OPTIONS *options, struct filterstate *ofilter,
+           unsigned int tm, int facilitytime, char *ifptr);
 void selectiface(char *ifname, int withall, int *aborted);
 void ifstats(const struct OPTIONS *options, int facilitytime);
 void detstats(char *iface, const struct OPTIONS *options, int facilitytime);
