@@ -63,16 +63,10 @@ extern int daemonized;
  * from log.c, applicable only to this module 
  */
 
-extern void writeethlog(struct ethtabent *list, int units, long nsecs,
+extern void writeethlog(struct ethtabent *list, int units, unsigned long nsecs,
 			FILE * logfile);
 extern char *ltrim(char *buf);
 
-/*
- * external declaration of sort status window routine
- */
- 
-extern void show_sort_statwin(WINDOW **win, PANEL **panel);
-
 /*
  * SIGUSR1 logfile rotation handler
  */
@@ -258,7 +252,7 @@ struct ethtabent *in_ethtable(struct ethtab *table, unsigned int linktype,
     return NULL;
 }
 
-void updateethent(struct ethtabent *entry, unsigned int pktsize,
+void updateethent(struct ethtabent *entry, int pktsize,
 		  int is_ip, int inout)
 {
     if (inout == 0) {
@@ -533,17 +527,107 @@ void swaphostents(struct ethtab *list, struct ethtabent *p1,
     p1->next_entry->next_entry = p2nextsaved;
 }
 
+unsigned long long ql_getkey(struct ethtabent *entry, int ch)
+{
+    unsigned long long result = 0;
+    
+    switch(ch) {
+    case 'P':
+        result = entry->next_entry->un.figs.inpcount;
+        break;
+    case 'I':
+        result = entry->next_entry->un.figs.inippcount;
+        break;
+    case 'B':
+        result = entry->next_entry->un.figs.inbcount;
+        break;
+    case 'K':
+        result = entry->next_entry->un.figs.outpcount;
+        break;
+    case 'O':
+        result = entry->next_entry->un.figs.outippcount;
+        break;
+    case 'Y':
+        result = entry->next_entry->un.figs.outbcount;
+        break;
+    }        
+    return result;
+}
+
+struct ethtabent *ql_partition(struct ethtab *table,
+                                 struct ethtabent **low,
+                                 struct ethtabent **high, int ch)
+{
+    struct ethtabent *pivot = *low;
+    
+    struct ethtabent *left = *low;
+    struct ethtabent *right = *high;
+    struct ethtabent *ptmp;
+    
+    unsigned long long pivot_value;
+    
+    pivot_value = ql_getkey(pivot, ch);
+    
+    while (left->index < right->index) {
+        while ((ql_getkey(left, ch) >= pivot_value)
+               && (left->next_entry->next_entry != NULL))
+            left = left->next_entry->next_entry;
+            
+        while (ql_getkey(right, ch) < pivot_value)
+            right = right->prev_entry->prev_entry;
+            
+        if (left->index < right->index) {
+            swaphostents(table, left, right);
+            
+            if (*low == left)
+                *low = right;
+                
+            if (*high == right)
+                *high = left;
+                
+	    ptmp = left;
+	    left = right;
+	    right = ptmp;
+	}
+    }
+    swaphostents(table, pivot, right);	    
+    
+    if (*low == pivot)
+        *low = right;
+                
+    if (*high == right)
+        *high = pivot;
+            
+    return pivot;
+}
+
+/*
+ * Quicksort routine for the LAN station monitor
+ */
+ 
+void quicksort_lan_entries(struct ethtab *table, struct ethtabent *low,
+                           struct ethtabent *high, int ch)
+{
+    struct ethtabent *pivot;
+    
+    if ((high == NULL) || (low == NULL))
+        return;
+        
+    if (high->index > low->index) {
+        pivot = ql_partition(table, &low, &high, ch);
+        
+        if (pivot->prev_entry != NULL)
+            quicksort_lan_entries(table, low, pivot->prev_entry->prev_entry, ch);
+        
+        quicksort_lan_entries(table, pivot->next_entry->next_entry, high, ch);
+    }
+}
+
 void sort_hosttab(struct ethtab *list, int *idx, int command)
 {
-    struct ethtabent *ptemp1, *ptemp2;
-    struct ethtabent *ptemp2next;
-
-    unsigned int long long val1 = 0, val2 = 0;
+    struct ethtabent *ptemp1;
     unsigned int idxtmp;
     
-    WINDOW *win;
-    PANEL *panel;
-
     if (!list->head)
 	return;
 
@@ -553,62 +637,8 @@ void sort_hosttab(struct ethtab *list, int *idx, int command)
        (command != 'K') && (command != 'O') && (command != 'Y'))
         return;
 
-    show_sort_statwin(&win, &panel);
-    ptemp1 = list->head;
+    quicksort_lan_entries(list, list->head, list->tail->prev_entry, command);
 
-    while (ptemp1 != NULL) {
-        ptemp1->un.desc.printed = 0;
-	ptemp2 = ptemp1->next_entry->next_entry;
-
-        mvwprintw(win, 2, 13, "%u%%", ptemp1->next_entry->index * 100 / list->tail->index);
-        update_panels();
-        doupdate();
-        
-	while (ptemp2 != NULL) {
-	    ptemp2next = ptemp2->next_entry->next_entry;
-
-	    switch (command) {
-	    case 'P':		/* order by packets in */
-		val1 = ptemp1->next_entry->un.figs.inpcount;
-		val2 = ptemp2->next_entry->un.figs.inpcount;
-		break;
-	    case 'I':		/* order by IP in */
-		val1 = ptemp1->next_entry->un.figs.inippcount;
-		val2 = ptemp2->next_entry->un.figs.inippcount;
-		break;
-	    case 'B':		/* order by bytes in */
-		val1 = ptemp1->next_entry->un.figs.inbcount;
-		val2 = ptemp2->next_entry->un.figs.inbcount;
-		break;
-	    case 'K':		/* order by Pkts out */
-		val1 = ptemp1->next_entry->un.figs.outpcount;
-		val2 = ptemp2->next_entry->un.figs.outpcount;
-		break;
-	    case 'O':		/* order by IP out */
-		val1 = ptemp1->next_entry->un.figs.outippcount;
-		val2 = ptemp2->next_entry->un.figs.outippcount;
-		break;
-	    case 'Y':		/* order by Bytes out */
-		val1 = ptemp1->next_entry->un.figs.outbcount;
-		val2 = ptemp2->next_entry->un.figs.outbcount;
-		break;
-	    }
-
-	    if (val1 < val2) {	/* descending sort */
-		swaphostents(list, ptemp1, ptemp2);
-		ptemp1 = ptemp2;
-	    }
-	    ptemp2 = ptemp2next;
-	}
-
-	ptemp1 = ptemp1->next_entry->next_entry;
-    }				/* end of the sorting */
-
-    del_panel(panel);
-    delwin(win);
-    update_panels();
-    doupdate();
-    
     ptemp1 = list->firstvisible = list->head;
     *idx = 1;
     idxtmp = 0;
