@@ -31,12 +31,13 @@ details.
 #include "error.h"
 #include "links.h"
 
-#define NUM_SUPPORTED_IFACES 9
+#define NUM_SUPPORTED_IFACES 11
 
 extern int daemonized;
 
 char ifaces[NUM_SUPPORTED_IFACES][6] =
-    { "lo", "eth", "sl", "ppp", "ippp", "plip", "fddi", "isdn", "dvb" };
+    { "lo", "eth", "sl", "ppp", "ippp", "plip", "fddi", "isdn", "dvb",
+      "pvc", "hdlc" };
 
 char *ltrim(char *buf)
 {
