@@ -27,7 +27,7 @@ void initfields(struct FIELDLIST *list, int leny, int lenx, int begy,
     list->fieldpanel = new_panel(list->fieldwin);
     stdwinset(list->fieldwin);
     wtimeout(list->fieldwin, -1);
-    wattrset(list->fieldwin, STDATTR);
+    wattrset(list->fieldwin, DLGTEXTATTR);
     colorwin(list->fieldwin);
     update_panels();
     doupdate();
