@@ -1,13 +1,14 @@
-/***
-
+/*
+===========================================================================
 IPTraf
 An IP Network Statistics Utility
 Written by Gerard Paul Java <riker@seul.org>
 Copyright (c) Gerard Paul Java 1997-2001
-   	
-Version 2.4.0
+
+Version 2.5.0
 Main Module
 
+---------------------------------------------------------------------------
 This software is open-source; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
@@ -17,8 +18,8 @@ This program is distributed WITHOUT ANY WARRANTY; without even the
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License in the included COPYING file for
 details.
-
-***/
+---------------------------------------------------------------------------
+*/
 
 #define MAIN_MODULE
 
@@ -34,12 +35,14 @@ details.
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <errno.h>
 #include "dirs.h"
 #include "deskman.h"
 #include "menurt.h"
 #include "stdwinset.h"
-#include "utfdefs.h"
-#include "othfilter.h"
+#include "fltmgr.h"
+#include "fltdefs.h"
+#include "fltselect.h"
 #include "utfilter.h"
 #include "ifstats.h"
 #include "serv.h"
@@ -68,6 +71,12 @@ char active_facility_countfile[64];
 
 extern void about();
 
+void press_enter_to_continue(void)
+{
+    fprintf(stderr, "Press Enter to continue.\n");
+    getchar();
+}
+
 void clearfiles(char *prefix, char *directory)
 {
     DIR *dir;
@@ -76,6 +85,13 @@ void clearfiles(char *prefix, char *directory)
     
     dir = opendir(directory);
     
+    if (dir == NULL) {
+        fprintf(stderr, "\nUnable to read dierctory %s\n%s\n",
+                directory, strerror(errno));
+        press_enter_to_continue();
+        return;
+    }
+    
     do {
         dir_entry = readdir(dir);
         if (dir_entry != NULL) {
@@ -203,10 +219,7 @@ void program_interface(struct OPTIONS *options,
     int break_row = 1;
     int aborted;
     int break_aborted;
-    int faborted;
-    int filtered = 0;
-    struct filterlist fl;
-    struct othpoptions ofilter;
+    struct filterstate ofilter;
     char ifname[10];
     char *ifptr = NULL;
     struct porttab *ports;
@@ -216,18 +229,16 @@ void program_interface(struct OPTIONS *options,
     attrset(STATUSBARATTR);
     mvprintw(0, 1, "IPTraf");
     
-    loadsavedtcpfilter(&fl, &filtered);
-    loadothfilter(&ofilter);
+    loadfilters(&ofilter);
     indicate("");
     loadaddports(&ports);
 
-
     if (opt == 0) {
         attrset(STATUSBARATTR);
         mvprintw(LINES - 1, 1, PLATFORM);
 	about();
 
-	initmenu(&menu, 14, 35, (LINES - 14) / 2, (COLS - 35) / 2);
+	initmenu(&menu, 13, 35, (LINES - 14) / 2, (COLS - 35) / 2);
 
 	additem(&menu, " IP traffic ^m^onitor",
 		"Displays current IP traffic information");
@@ -235,17 +246,15 @@ void program_interface(struct OPTIONS *options,
 		"Displays some statistics for attached interfaces");
 	additem(&menu, " ^D^etailed interface statistics",
 		"Displays more statistics for a selected interface");
-	additem(&menu, " Statistical ^b^reakdowns",
+	additem(&menu, " Statistical ^b^reakdowns...",
 		"Facilities for traffic counts by packet size or TCP/UDP port");
 	additem(&menu, " ^L^AN station monitor",
 		"Displays statistics on detected LAN stations");
 	additem(&menu, NULL, NULL);
-	additem(&menu, " ^T^CP display filters",
-		"Manages TCP display filters");
-	additem(&menu, " Other ^p^rotocol filters",
-		"Select which non-TCP protocols to display");
+        additem(&menu, " ^F^ilters...",
+		"Allows you to select traffic display and logging criteria");
 	additem(&menu, NULL, NULL);
-	additem(&menu, " C^o^nfigure", "Set various program options");
+	additem(&menu, " C^o^nfigure...", "Set various program options");
 	additem(&menu, NULL, NULL);
 	additem(&menu, " E^x^it", "Exits program");
 
@@ -265,7 +274,7 @@ void program_interface(struct OPTIONS *options,
 			ifptr = NULL;
 
 		    ipmon(options,
-			  filtered, &fl, &ofilter, options->timeout, 0,
+			  &ofilter, options->timeout, 0,
 			  ifptr);
 		}
 		break;
@@ -310,17 +319,14 @@ void program_interface(struct OPTIONS *options,
 		}
 		break;
 	    case 7:
-		tcpfilterselect(&fl, &filtered, &faborted);
+		config_filters(&ofilter);
+		savefilters(&ofilter);
 		break;
-	    case 8:
-		othfilterselect(&ofilter);
-		saveothfilter(&ofilter);
-		break;
-	    case 10:
+	    case 9:
 		setoptions(options, &ports);
 		saveoptions(options);
 		break;
-	    case 12:
+	    case 11:
 		endloop = 1;
 		break;
 	    }
@@ -337,7 +343,7 @@ void program_interface(struct OPTIONS *options,
 		ifptr = optarg;
 
 	    ipmon(options,
-		  filtered, &fl, &ofilter, options->timeout, facilitytime,
+		  &ofilter, options->timeout, facilitytime,
 		  ifptr);
 	    break;
 	case 'g':
@@ -436,8 +442,7 @@ void mark_first_instance()
     fd = open(IPTIDFILE, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR);
     if (fd < 0) {
 	fprintf(stderr, "\nWarning: unable to tag this process\r\n");
-	fprintf(stderr, "Press Enter to continue\r\n");
-	getchar();
+        press_enter_to_continue();
 	return;
     }
     close(fd);
