@@ -28,9 +28,9 @@ details.
 #include "ifaces.h"
 #include "ipcsum.h"
 #include "promisc.h"
-#include "utfdefs.h"
+#include "fltdefs.h"
 #include "utfilter.h"
-#include "othfilter.h"
+#include "fltselect.h"
 #include "deskman.h"
 #include "error.h"
 #include "attrs.h"
@@ -54,7 +54,6 @@ void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
 		 unsigned int pktlen, int mac, char *message);
 void write_tcp_unclosed(int logging, FILE * fd, struct tcptable *table);
 void write_timeout_log(int logging, FILE *logfile, struct tcptableent *tcpnode);
-void show_sort_statwin(WINDOW **statwin, PANEL **statpanel);
 
 void rotate_ipmon_log()
 {
@@ -284,7 +283,7 @@ void show_tcpsort_win(WINDOW ** win, PANEL ** panel)
 
 /*
  * Routine to swap two TCP entries.  p1 and p2 are pointers to TCP entries,
- * but p1 must be ahead than p2.  It's a linked list thing.
+ * but p1 must be ahead of p2.  It's a linked list thing.
  */
 void swap_tcp_entries(struct tcptable *table,
 		      struct tcptableent *p1, struct tcptableent *p2)
@@ -330,27 +329,131 @@ void swap_tcp_entries(struct tcptable *table,
     p1->next_entry->next_entry = p2nextsaved;
 }
 
-/* 
- * Sort TCP window entries.  Contributed by Gal Laszlo.  Edited and
- * modified, GPJ <riker@seul.org>
- * Contributor's note: by LG - Gal Laszlo ( SlowTCP@hotmail.com ) 2000.07.19.
- * Author's note: Special (accented) characters removed from contributed
- *                source.
+unsigned long long qt_getkey(struct tcptableent *entry, int ch)
+{
+    if (ch == 'B')
+        return (max(entry->bcount, entry->oth_connection->bcount));
+        
+    return (max(entry->pcount, entry->oth_connection->pcount));
+}
+
+struct tcptableent *qt_partition(struct tcptable *table,
+                                 struct tcptableent **low,
+                                 struct tcptableent **high,
+                                 int ch, time_t timeout, int logging,
+                                 FILE *logfile, int *nomem)
+{
+    struct tcptableent *pivot = *low;
+    
+    struct tcptableent *left = *low;
+    struct tcptableent *right = *high;
+    struct tcptableent *ptmp;
+    
+    unsigned long long pivot_value;
+    
+    time_t now;
+    
+    pivot_value = qt_getkey(pivot, ch);
+    
+    now = time(NULL);
+    
+    while (left->index < right->index) {
+        while ((qt_getkey(left, ch) >= pivot_value)
+               && (left->next_entry->next_entry != NULL)) {
+            
+            /*
+             * Might as well check out timed out entries here too.
+             */   
+            if ((timeout > 0) && ((now - left->lastupdate) / 60 > timeout) &&
+               (!(left->inclosed))) {
+                left->timedout = left->oth_connection->timedout = 1;
+                addtoclosedlist(table, left, nomem);
+                
+                if (logging)
+                    write_timeout_log(logging, logfile, left);
+            }
+            
+            left = left->next_entry->next_entry;
+        }
+                    
+        while (qt_getkey(right, ch) < pivot_value) {
+            /*
+             * Might as well check out timed out entries here too.
+             */   
+            if ((timeout > 0) && ((now - right->lastupdate) / 60 > timeout) &&
+               (!(right->inclosed))) {
+                right->timedout = right->oth_connection->timedout = 1;
+                addtoclosedlist(table, right, nomem);
+                
+                if (logging)
+                    write_timeout_log(logging, logfile, right);
+            }
+            right = right->prev_entry->prev_entry;
+        }
+        
+        if (left->index < right->index) {
+            swap_tcp_entries(table, left, right);
+            
+            if (*low == left)
+                *low = right;
+                
+            if (*high == right)
+                *high = left;
+                
+	    ptmp = left;
+	    left = right;
+	    right = ptmp;
+	}
+    }
+    swap_tcp_entries(table, pivot, right);	    
+    
+    if (*low == pivot)
+        *low = right;
+                
+    if (*high == right)
+        *high = pivot;
+            
+    return pivot;
+}
+
+/*
+ * Quicksort the TCP entries.
  */
+void quicksort_tcp_entries(struct tcptable *table, struct tcptableent *low,
+                           struct tcptableent *high, int ch, time_t timeout,
+                           int logging, FILE *logfile, int *nomem)
+{
+    struct tcptableent *pivot;
+    
+    if ((high == NULL) || (low == NULL))
+        return;
+        
+    if (high->index > low->index) {
+        pivot = qt_partition(table, &low, &high, ch, timeout,
+                             logging, logfile, nomem);
+        
+        if (pivot->prev_entry != NULL)
+            quicksort_tcp_entries(table, low, pivot->prev_entry->prev_entry,
+                                  ch, timeout, logging, logfile, nomem);
+        
+        quicksort_tcp_entries(table, pivot->next_entry->next_entry, high,
+                              ch, timeout, logging, logfile, nomem);
+    }
+}
+
+/* 
+ * This function sorts the TCP window.  The old exchange sort has been
+ * replaced with a Quicksort algorithm.
+ */
+ 
 void sortipents(struct tcptable *table, unsigned long *idx, int ch,
 		int mode, int logging, FILE *logfile,
 		time_t timeout, int *nomem)
 {
-    struct tcptableent *tcptmp1, *tcptmp2, *tcp_next2;
-    unsigned long long val1 = 0, val2 = 0;
-    unsigned long lastidx;
+    struct tcptableent *tcptmp1;
     unsigned int idxtmp;
-    time_t now;
-    WINDOW *statwin;
-    PANEL *panel;
     
-    if ((table->head == NULL)
-	|| (table->head->next_entry->next_entry == NULL))
+    if ((table->head == NULL) || (table->head->next_entry->next_entry == NULL))
         return;
 
     ch = toupper(ch);
@@ -358,67 +461,13 @@ void sortipents(struct tcptable *table, unsigned long *idx, int ch,
     if ((ch != 'P') && (ch != 'B'))
         return;
             
-    show_sort_statwin(&statwin, &panel);
-    
-    lastidx = table->tail->index;
-    
-    tcptmp1 = table->head;
-
-    now = time(NULL);
-    
-    while (tcptmp1 != NULL) {
-	tcptmp2 = tcptmp1->next_entry->next_entry;
-	/*
-	 * While we're examining, might as well mark any timed
-	 * out entries too.
-	 */
-	 
-        if ((timeout > 0) && (tcptmp1->lastupdate / 60 > timeout) && !(tcptmp1->inclosed)) {
-            tcptmp1->timedout = tcptmp1->oth_connection->timedout = 1;
-            addtoclosedlist(table, tcptmp1, nomem);
-            if (logging)
-                write_timeout_log(logging, logfile, tcptmp1);
-        }
-    
-        mvwprintw(statwin, 2, 13, "%u%%", tcptmp1->next_entry->index * 100 / lastidx);
-        update_panels();
-        doupdate();
-             
-        /*
-         * Inner loop
-         */
-	while (tcptmp2 != NULL) {
-	    tcp_next2 = tcptmp2->next_entry->next_entry;
-	    switch (ch) {
-	    case 'B':
-		val1 =
-		    max(tcptmp1->bcount, tcptmp1->oth_connection->bcount);
-		val2 =
-		    max(tcptmp2->bcount, tcptmp2->oth_connection->bcount);
-		break;
-	    case 'P':
-		val1 =
-		    max(tcptmp1->pcount, tcptmp1->oth_connection->pcount);
-		val2 =
-		    max(tcptmp2->pcount, tcptmp2->oth_connection->pcount);
-		break;
-	    }
-	    if (val1 < val2) {	/* descending sort */
-		swap_tcp_entries(table, tcptmp1, tcptmp2);
-		tcptmp1 = tcptmp2;
-	    }
-
-	    tcptmp2 = tcp_next2;
-	}
-
-	tcptmp1 = tcptmp1->next_entry->next_entry;
-    }
+    quicksort_tcp_entries(table, table->head, table->tail->prev_entry,
+                          ch, timeout, logging, logfile, nomem);
      
-    del_panel(panel);
-    delwin(statwin);
     update_panels();
     doupdate();
     colorwin(table->tcpscreen);
+
     tcptmp1 = table->firstvisible = table->head;
     *idx = 1;
     idxtmp = 0;
@@ -481,8 +530,8 @@ int checkrvnamed(void)
  * The IP Traffic Monitor
  */
 
-void ipmon(const struct OPTIONS *options, int filtered,
-	   struct filterlist *fl, struct othpoptions *ofilter,
+void ipmon(const struct OPTIONS *options,
+	   struct filterstate *ofilter,
 	   unsigned int tcptm, int facilitytime, char *ifptr)
 {
     int logging = options->logging;
@@ -564,6 +613,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
     int rvnfd = 0;
 
     int instance_id;
+    int revlook = options->revlook;
     
     /* 
      * Mark this instance of the traffic monitor
@@ -616,7 +666,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
     init_tcp_table(&table);
     init_othp_table(&othptbl, options->mac);
 
-    if (options->revlook) {
+    if (revlook) {
 	if (checkrvnamed())
 	    open_rvn_socket(&rvnfd);
     } else
@@ -639,6 +689,8 @@ void ipmon(const struct OPTIONS *options, int filtered,
     update_panels();
     doupdate();
 
+    bzero(&isdntable, sizeof(struct isdntab));
+    
     /*
      * Try to open log file if logging activated.  Turn off logging
      * (for this session only) if an error was discovered in opening
@@ -646,8 +698,6 @@ void ipmon(const struct OPTIONS *options, int filtered,
      * situation may be corrected later.
      */
 
-    bzero(&isdntable, sizeof(struct isdntab));
-    
     if (logging) {
         if (strcmp(current_logfile, "") == 0)
             strncpy(current_logfile, gen_instance_logname(IPMONLOG, instance_id), 80);
@@ -767,16 +817,22 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		case 24:
 		case 27:
 		    exitloop = 1;
+		    break;
 		}
 	    } else if (keymode == 1) {
 		keymode = 0;
 		del_panel(sortpanel);
 		delwin(sortwin);
+		show_sort_statwin(&sortwin, &sortpanel);
 		update_panels();
 		doupdate();
 		sortipents(&table, &screen_idx, ch, mode, logging, logfile,
 		           options->timeout, &nomem);
-	    }
+		del_panel(sortpanel);
+		delwin(sortwin);
+		update_panels();
+		doupdate();
+            }
 	}
 	if (readlen > 0) {
 	    if (!iface_supported(ifname))
@@ -884,8 +940,8 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		 * Apply TCP filter, if active.
 		 */
 
-		if ((filtered) &&
-		    (!(utfilter(fl, ippacket->saddr, ippacket->daddr,
+		if ((ofilter->tfiltercode) &&
+		    (!(utfilter(&(ofilter->tfl), ippacket->saddr, ippacket->daddr,
 				ntohs(sport), ntohs(dport), IPPROTO_TCP)))) {
 		    show_ip_stats(statwin, iptotal, tcptotal, udptotal,
 				  icmptotal, noniptotal);
@@ -897,18 +953,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
 			     logging, logfile, &nomem);
 
 		/* 
-		 * Add a new entry if it doesn't exist, and if it is a 
-		 * SYN packet or any packet with more than just headers,
-		 * and, to reduce the chances of stales, not a FIN.
+		 * Add a new entry if it doesn't exist, and, 
+		 * to reduce the chances of stales, not a FIN.
 		 */
 
 		if ((ntohs(ippacket->frag_off) & 0x3fff) == 0) {	/* first frag only */
 		    totalhlen = iphlen + transpacket->doff * 4;
 
-		    if ((tcpentry == NULL) && (!(transpacket->fin)) &&
-			((totalhlen < ntohs(ippacket->tot_len)) ||
-			 ((totalhlen == ntohs(ippacket->tot_len)) &&
-			  (transpacket->syn)))) {
+		    if ((tcpentry == NULL) && (!(transpacket->fin))) {
 
 			/*
 			 * Ok, so we have a packet.  Add it if this connection
@@ -922,7 +974,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 						(unsigned long)
 						ippacket->daddr, sport,
 						dport, ippacket->protocol,
-						ifname, options->revlook,
+						ifname, &revlook,
 						rvnfd, options->servnames,
 						&nomem);
 
@@ -947,14 +999,14 @@ void ipmon(const struct OPTIONS *options, int filtered,
 		     */
 
 		    if (!(tcpentry->stat & FLAG_RST)) {
-			if (options->revlook) {
+			if (revlook) {
 			    p_sstat = tcpentry->s_fstat;
 			    p_dstat = tcpentry->d_fstat;
 			}
 			updateentry(&table, tcpentry, transpacket, tpacket,
 			            linktype, nbplen,
 				    br, ippacket->frag_off, logging,
-				    options->mac, options->revlook, rvnfd, logfile,
+				    options->mac, &revlook, rvnfd, logfile,
 				    &nomem);
 
 			/*
@@ -973,7 +1025,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 				    tcpentry, nbplen, options->mac, msgstring);
 		        }
 			
-			if ((options->revlook)
+			if ((revlook)
 			    && (((p_sstat != RESOLVED)
 				 && (tcpentry->s_fstat == RESOLVED))
 				|| ((p_dstat != RESOLVED)
@@ -997,7 +1049,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 			if (((tcpentry->oth_connection->finsent == 2) &&	/* FINed and ACKed */
 			     (ntohl(transpacket->seq) ==
 			      tcpentry->oth_connection->finack))
-			    || ((options->revlook)
+			    || ((revlook)
 				&&
 				(((p_sstat
 				   != RESOLVED)
@@ -1049,7 +1101,7 @@ void ipmon(const struct OPTIONS *options, int filtered,
 				       ippacket->daddr, IS_IP,
 				       ippacket->protocol, linktype,
 				       (char *) tpacket, (char *) transpacket,
-				       nbplen, ifname, options->revlook,
+				       nbplen, ifname, &revlook,
 				       rvnfd, tcptm, logging, logfile,
 				       options->servnames, fragment,
 				       &nomem);
@@ -1136,7 +1188,9 @@ void ipmon(const struct OPTIONS *options, int filtered,
     }
     adjust_instance_count(PROCCOUNTFILE, -1);
     adjust_instance_count(ITRAFMONCOUNTFILE, -1);
-
+    
+    attrset(STDATTR);
+    mvprintw(0, COLS - 20, "                    ");
     del_panel(table.tcppanel);
     del_panel(table.borderpanel);
     del_panel(othptbl.othppanel);
