@@ -27,6 +27,7 @@ details.
 #include <unistd.h>
 #include <fcntl.h>
 #include <net/if_arp.h>
+#include <ctype.h>
 #include "deskman.h"
 #include "stdwinset.h"
 #include "menurt.h"
@@ -44,14 +45,38 @@ void etherr(void)
     errbox("Unable to open host description file", ANYKEY_MSG, &resp);
 }
 
+void add_desclist_node(struct desclist *list, struct desclistent *ptmp)
+{
+	if (list->head == NULL) {
+	    list->head = ptmp;
+	    ptmp->prev_entry = NULL;
+	} else {
+	    list->tail->next_entry = ptmp;
+	    ptmp->prev_entry = list->tail;
+	}
+
+	ptmp->next_entry = NULL;
+	list->tail = ptmp;
+}
+
+/*
+ * Loads descriptions from the IPTraf LAN host description files.
+ * Now also loads /etc/ethers as well.
+ *
+ * In case of a duplicate in the IPTraf definition files and /etc/ethers,
+ * the IPTraf definition files take precedence.
+ */
+ 
 void loaddesclist(struct desclist *list, unsigned int linktype)
 {
     struct desclistent *ptmp = NULL;
     FILE *fd = NULL;
     char descline[140];
     char *desctoken;
-
-    bzero(list, sizeof(list));
+    char etherline[140];
+    int i,j;   /* counters used when parsing /etc/ethers */
+	
+    bzero(list, sizeof(struct desclist));
 
     if (linktype == LINK_ETHERNET)
 	fd = fopen(ETHFILE, "r");
@@ -82,17 +107,84 @@ void loaddesclist(struct desclist *list, unsigned int linktype)
 	    strncpy(ptmp->rec.desc, desctoken, 64);
 	else
 	    strcpy(ptmp->rec.desc, "");
+        
+        add_desclist_node(list, ptmp);
+    }
 
-	if (list->head == NULL) {
-	    list->head = ptmp;
-	    ptmp->prev_entry = NULL;
-	} else {
-	    list->tail->next_entry = ptmp;
-	    ptmp->prev_entry = list->tail;
+    fclose(fd);
+    
+    /* 
+     * Loads MAC addresses defined in /etc/ethers.  Contributed by
+     * Debian maintainter Frederic Peters <fpeters@debian.org>.  Thanks
+     * Frederic!
+     *
+     * Contributor's note:
+     *  loading other ethenet mac addresses from /etc/ethers (used by tcpdump)
+     *
+     * Author's note:
+     *  Moved significantly repeating code to a function.
+     */
+    if (linktype != LINK_ETHERNET)
+        return;
+            
+    fd = fopen("/etc/ethers", "r");
+    
+    if (fd == NULL)
+	return;
+    
+    while (!feof(fd)) {
+	ptmp = malloc(sizeof(struct desclistent));
+	if (ptmp == NULL) {
+	    printnomem();
+	    return;
 	}
+	bzero(ptmp, sizeof(struct desclistent));
+	bzero(descline, 140);
+	bzero(etherline, 140);
+	fgets(etherline, 140, fd);
 
-	ptmp->next_entry = NULL;
-	list->tail = ptmp;
+	/* 
+	 * Convert /etc/ethers line to a descline
+	 */
+	if (etherline[0] == '#' || etherline[0] == '\n' || etherline[0] == 0) {
+	    free(ptmp);
+	    continue;
+	}
+	
+	if (strchr(etherline, '\n') )
+	    strchr(etherline, '\n')[0] = 0;
+	
+	j = 0;
+	for (i = 0; i < 20 && !isspace(etherline[i]); i++) {
+	    if (etherline[i] == ':')
+	        continue;
+	    descline[j++] = tolower(etherline[i]);
+	}
+	descline[j] = ':';
+	
+	/* 
+	 * Skip over whitespace between MAC address and IP addr/host name
+	 */
+	 
+	while (isspace(etherline[i++]))
+	    ;
+	   
+	strncat(descline, etherline + i - 1, 130);
+	
+	if (strcmp(descline, "") == 0) {
+	    free(ptmp);
+	    continue;
+	}
+	
+	strncpy(ptmp->rec.address, strtok(descline, ":"), 12);
+	desctoken = strtok(NULL, "\n");
+
+	if (desctoken != NULL)
+	    strncpy(ptmp->rec.desc, desctoken, 64);
+	else
+	    strcpy(ptmp->rec.desc, "");
+
+        add_desclist_node(list, ptmp);
     }
 
     fclose(fd);
@@ -281,7 +373,7 @@ void descdlg(struct descrec *rec, char *initaddr, char *initdesc,
     win = newwin(8, 70, 8, (COLS - 70) / 2);
     panel = new_panel(win);
 
-    wattrset(win, BOXATTR);
+    wattrset(win, DLGBOXATTR);
     colorwin(win);
     box(win, ACS_VLINE, ACS_HLINE);
     wmove(win, 6, 2 * COLS / 80);
@@ -289,7 +381,7 @@ void descdlg(struct descrec *rec, char *initaddr, char *initdesc,
     wmove(win, 6, 20 * COLS / 80);
     stdkeyhelp(win);
 
-    wattrset(win, STDATTR);
+    wattrset(win, DLGTEXTATTR);
     wmove(win, 2, 2 * COLS / 80);
     wprintw(win, "MAC Address:");
     wmove(win, 4, 2 * COLS / 80);
