@@ -6,5 +6,7 @@
 #define LINK_ISDN_RAWIP		6
 #define LINK_ISDN_CISCOHDLC	7
 #define LINK_FDDI		8
+#define LINK_FRAD		9
+#define LINK_DLCI		10
 #define LINK_INVALID		0
 
