@@ -24,6 +24,8 @@ details.
 #include <netinet/in.h>
 #include <linux/if_ether.h>
 #include <net/if_arp.h>
+#include "attrs.h"
+#include "deskman.h"
 #include "dirs.h"
 #include "error.h"
 #include "tcptable.h"
@@ -35,13 +37,15 @@ details.
 #include "links.h"
 #include "mode.h"
 #include "options.h"
+#include "input.h"
 
 #define MSGSTRING_MAX		240
 #define TARGET_LOGNAME_MAX	160
 #define TIME_TARGET_MAX		30
 
-volatile int rotate_flag;
+int rotate_flag;
 char target_logname[TARGET_LOGNAME_MAX];
+char current_logfile[TARGET_LOGNAME_MAX];
 
 void openlogerr()
 {
@@ -49,6 +53,61 @@ void openlogerr()
     errbox("Unable to open log file", ANYKEY_MSG, &resp);
 }
 
+/*
+ * Generates a log file based on a template for a particular instance of
+ * a facility.   Used by the IP Traffic Monitor and LAN Station Monitor.
+ */
+ 
+char *gen_instance_logname(char *template, int instance_num)
+{
+    static char filename[80];
+    
+    snprintf(filename, 80, "%s-%d.log", template, instance_num);
+    return filename;
+}
+
+void input_logfile(char *target, int *logging)
+{
+    WINDOW *dlgwin;
+    PANEL *dlgpanel;
+    struct FIELDLIST fieldlist;
+    int aborted;
+        
+    dlgwin = newwin(11, 60, (LINES - 11) / 2, (COLS - 60) / 2);
+    dlgpanel = new_panel(dlgwin);
+    
+    wattrset(dlgwin, DLGBOXATTR);
+    colorwin(dlgwin);
+    box(dlgwin, ACS_VLINE, ACS_HLINE);
+    mvwprintw(dlgwin, 0, 1, " Logging Enabled ");
+    wattrset(dlgwin, DLGTEXTATTR);
+    mvwprintw(dlgwin, 2, 2, "Enter the name of the file to which to write the log.");
+    mvwprintw(dlgwin, 4, 2, "If you don't specify a path, the log file will");
+    mvwprintw(dlgwin, 5, 2, "be placed in %s.", LOGDIR);
+    wmove(dlgwin, 9, 2);
+    stdkeyhelp(dlgwin);
+    wprintw(dlgwin, " (turns logging off)");
+    
+    initfields(&fieldlist, 1, 50, (LINES - 1) / 2 + 2, (COLS - 50) / 2 - 3);
+    addfield(&fieldlist, 48, 0, 0, target);
+    fillfields(&fieldlist, &aborted);
+    
+    if (!aborted) {
+        if (strchr(fieldlist.list->buf, '/') == NULL)
+            snprintf(target, 48, "%s/%s", LOGDIR, fieldlist.list->buf);
+        else
+            strncpy(target, fieldlist.list->buf, 48);
+    }
+    
+    *logging = !aborted;
+        
+    destroyfields(&fieldlist);
+    del_panel(dlgpanel);
+    delwin(dlgwin);
+    update_panels();
+    doupdate();
+}
+ 
 void opentlog(FILE ** fd, char *logfilename)
 {
     *fd = fopen(logfilename, "a");
@@ -91,15 +150,21 @@ void write_daemon_err(char *msg)
 }
 
 void writetcplog(int logging, FILE * fd, struct tcptableent *entry,
-		 unsigned int pktlen, char *message)
+		 unsigned int pktlen, int mac, char *message)
 {
     char msgbuf[MSGSTRING_MAX];
 
     if (logging) {
-	sprintf(msgbuf, "TCP; %s; %u bytes; from %s:%s to %s:%s; %s",
+        if (mac) {
+	    snprintf(msgbuf, MSGSTRING_MAX, "TCP; %s; %u bytes; from %s:%s to %s:%s (source MAC addr %s); %s",
+		entry->ifname, pktlen, entry->s_fqdn, entry->s_sname,
+		entry->d_fqdn, entry->d_sname, entry->smacaddr, message);
+	} else {
+	    snprintf(msgbuf, MSGSTRING_MAX, "TCP; %s; %u bytes; from %s:%s to %s:%s; %s",
 		entry->ifname, pktlen, entry->s_fqdn, entry->s_sname,
 		entry->d_fqdn, entry->d_sname, message);
-
+        }
+        
 	writelog(logging, fd, msgbuf);
     }
 }
@@ -139,6 +204,12 @@ void writeothplog(int logging, FILE * fd, char *protname,
 	strcat(msgbuffer, entry->iface);
 	sprintf(scratchpad, "; %u bytes;", entry->pkt_length);
 	strcat(msgbuffer, scratchpad);
+	
+	if ((entry->smacaddr[0] != '\0') && (withmac)) {
+	    sprintf(scratchpad, " source MAC address %s;", entry->smacaddr);
+	    strcat(msgbuffer, scratchpad);
+	}
+	
 	if (is_ip) {
 	    if (((entry->protocol == IPPROTO_UDP) && (!(entry->fragment)))
 		|| (entry->protocol == IPPROTO_TCP))
