@@ -6,6 +6,8 @@ Copyright (c) Gerard Paul Java 1997
 
 ***/
 
+char *gen_instance_logname(char *template, int instance_id);
+void input_logfile(char *target, int *aborted);
 void opentlog(FILE **fd, char *logfilename);
 void writelog(int logging, FILE * fd, char *msg);
 void write_daemon_err(char *msg);
