@@ -56,10 +56,10 @@ void additem(struct MENU *menu, char *item, char *desc)
     struct ITEM *tnode;
     char cur_option[OPTIONSTRLEN_MAX];
     char thekey[2];
-    
+
     if (menu->itemcount >= 25)
-        return;
-        
+	return;
+
     tnode = malloc(sizeof(struct ITEM));
 
     if (item != NULL) {
@@ -174,7 +174,9 @@ void menumoveto(struct MENU *menu, struct ITEM **itemptr, unsigned int row)
     *itemptr = tnode;
 }
 
-/* actually to the menu operation after all the initialization */
+/* 
+ * Actually do the menu operation after all the initialization
+ */
 
 void operatemenu(struct MENU *menu, int *position, int *aborted)
 {
@@ -248,7 +250,8 @@ void operatemenu(struct MENU *menu, int *position, int *aborted)
 	    break;		/* ignore caret key */
 	default:
 	    keyptr = strchr(menu->shortcuts, toupper(ch));
-	    if ((keyptr != NULL) && keyptr - menu->shortcuts < menu->itemcount) {
+	    if ((keyptr != NULL)
+		&& keyptr - menu->shortcuts < menu->itemcount) {
 		row = keyptr - menu->shortcuts + 1;
 		exitloop = 1;
 	    }
