@@ -7,9 +7,17 @@
 #define SELECTED 1
 #define NOTSELECTED 0
 
+#define SEPARATOR 0
+#define REGULARITEM 1
+
+#define OPTIONSTRLEN_MAX 50
+#define DESCSTRLEN_MAX 81
+#define SHORTCUTSTRLEN_MAX 25
+
 struct ITEM {
-    char option[39];
-    char desc[80];
+    char option[OPTIONSTRLEN_MAX];
+    char desc[DESCSTRLEN_MAX];
+    unsigned int itemtype;
     struct ITEM *prev;
     struct ITEM *next;
 };
@@ -22,11 +30,12 @@ struct MENU {
     int postn;
     int x1, y1;
     int x2, y2;
+    unsigned int menu_maxx;
     WINDOW *menuwin;
     PANEL *menupanel;
     WINDOW *descwin;
     PANEL *descpanel;
-    char shortcuts[22];
+    char shortcuts[SHORTCUTSTRLEN_MAX];
 };
 
 extern void initmenu(struct MENU *menu, int y1, int x1, int y2, int x2);
