@@ -57,16 +57,16 @@ void makeoptionmenu(struct MENU *menu)
     additem(menu, " Source ^M^AC addrs in traffic monitor",
             "Toggles display of source MAC addresses in the IP Traffic Monitor");
     additem(menu, NULL, NULL);
-    additem(menu, " ^T^imers", "Configures timeouts and intervals");
+    additem(menu, " ^T^imers...", "Configures timeouts and intervals");
     additem(menu, NULL, NULL);
     additem(menu, " ^A^dditional ports...",
 	    "Allows you to add port numbers higher than 1023 for the service stats");
     additem(menu, " ^D^elete port/range...",
 	    "Deletes a port or range of ports earlier added");
     additem(menu, NULL, NULL);
-    additem(menu, " ^E^thernet/PLIP host descriptions",
+    additem(menu, " ^E^thernet/PLIP host descriptions...",
 	    "Manages descriptions for Ethernet/PLIP addresses");
-    additem(menu, " ^F^DDI host descriptions",
+    additem(menu, " ^F^DDI host descriptions...",
 	    "Manages descriptions for FDDI addresses");
     additem(menu, NULL, NULL);
     additem(menu, " E^x^it configuration", "Returns to main menu");
@@ -76,7 +76,7 @@ void maketimermenu(struct MENU *menu)
 {
     initmenu(menu, 7, 35, (LINES - 19) / 2 + 7, (COLS - 35) / 2);
     additem(menu, " TCP ^t^imeout...",
-	    "Determines how long TCP entries may remain idle");
+	    "Sets the length of time before inactive TCP entries are considered idle");
     additem(menu, " ^L^ogging interval...",
 	    "Sets the time between loggings for interface, host, and service stats");
     additem(menu, " ^S^creen update interval...",
