@@ -108,7 +108,7 @@ struct othptabent *add_othp_entry(struct othptable *table,
 				  int is_ip, int protocol,
 				  unsigned short linkproto, char *packet,
 				  char *packet2, unsigned int br,
-				  char *ifname, int rev_lookup, int rvnfd,
+				  char *ifname, int *rev_lookup, int rvnfd,
 				  unsigned int tm, int logging,
 				  FILE * logfile, int servnames,
 				  int fragment, int *nomem)
