@@ -104,6 +104,12 @@ unsigned short getlinktype(unsigned short family, char *ifname,
     case ARPHRD_FDDI:
 	result = LINK_FDDI;
 	break;
+    case ARPHRD_FRAD:
+        result = LINK_FRAD;
+        break;
+    case ARPHRD_DLCI:
+        result = LINK_DLCI;
+        break; 
     default:
 	result = LINK_INVALID;
 	break;
@@ -136,6 +142,8 @@ void adjustpacket(char *tpacket, unsigned short family,
 	*packet = tpacket;
 	break;
     case LINK_ISDN_CISCOHDLC:
+    case LINK_FRAD:
+    case LINK_DLCI:
 	*packet = tpacket + 4;
 	*readlen -= 4;
 	break;
