@@ -103,18 +103,25 @@ int initialize_brackets(char *ifname, struct ifstat_brackets *brackets,
 	wprintw(win, "%8lu", 0);
     }
 
-    /* TODO? : Adjust values for !80 column displays */
     for (i = 10; i <= 19; i++) {
 	wattrset(win, STDATTR);
 	wmove(win, (i - 10) + 5, 36);
-	wprintw(win, "%4u to %4u:", brackets[i].floor, brackets[i].ceil);
+	
+	if (i != 19)
+	    wprintw(win, "%4u to %4u:", brackets[i].floor, brackets[i].ceil);
+        else
+	    wprintw(win, "%4u to %4u+:", brackets[i].floor, brackets[i].ceil);
+        
 	wmove(win, (i - 10) + 5, 57);
 	wattrset(win, HIGHATTR);
 	wprintw(win, "%8lu", 0);
     }
 
     wattrset(win, STDATTR);
-    mvwprintw(win, 17, 1, "Interface MTU is %d bytes", ifr.ifr_mtu);
+    mvwprintw(win, 17, 1, "Interface MTU is %d bytes, not counting the data-link header", ifr.ifr_mtu);
+    mvwprintw(win, 18, 1, "Maximum packet size is the MTU plus the data-link header length");
+    mvwprintw(win, 19, 1, "Packet size computations include data-link headers, if any");
+   
     *mtu = ifr.ifr_mtu;
     return 0;
 }
@@ -218,10 +225,7 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname,
 
     mvwprintw(win, 1, 1, "Packet size brackets for interface %s", ifname);
     wattrset(win, BOXATTR);
-    /* TODO : Adjust values for !80 column displays */
-/*
-   Packet Size (bytes)      Count     Packet Size (bytes)     Count");
- */
+
     mvwprintw(win, 4, 1, "Packet Size (bytes)");
     mvwprintw(win, 4, 26, "Count");
     mvwprintw(win, 4, 36, "Packet Size (bytes)");
@@ -305,7 +309,7 @@ void packet_size_breakdown(struct OPTIONS *options, char *ifname,
 	    updtime_usec = unow;
 	}
 	if (now - timeint >= 5) {
-	    printelapsedtime(starttime, now, 22, 1, borderwin);
+	    printelapsedtime(starttime, now, LINES - 3, 1, borderwin);
 	    timeint = now;
 	}
 	if ((now - startlog >= options->logspan) && (logging)) {
