@@ -3,10 +3,10 @@
 rvnamed		- reverse DNS lookup daemon for the IPTraf network
 		  statistics utility.
 
-Version 2.4.0                      Parallel with IPTraf 2.4
+Version 2.5.0                      Parallel with IPTraf 2.5
 
 Written by Gerard Paul Java
-Copyright (c) Gerard Paul Java 1998
+Copyright (c) Gerard Paul Java 1998-2001
 
 rvnamed is a daemon designed to do reverse DNS lookups, but return the
 IP address immediately while the lookup goes on in the background.
@@ -190,7 +190,8 @@ int main(void)
     FILE *logfile;
 
     extern int errno;
-
+    pid_t pid;
+    
     /* Daemonization Sequence */
 
 #ifndef DEBUG
@@ -364,22 +365,31 @@ int main(void)
 
 			if (addrstat(&rvnpacket, hostlist, lastfree) ==
 			    NOTRESOLVED) {
-			    hostlist[hostindex].addr =
-				rvnpacket.saddr.s_addr;
-			    hostlist[hostindex].ready = RESOLVING;
-
-			    hostindex++;
-
-			    if (hostindex == NUM_CACHE_ENTRIES)
-				hostindex = 0;
-
-			    if (lastfree < NUM_CACHE_ENTRIES)
-				lastfree++;
-
 			    fflush(logfile);	/* flush all data prior */
 			    			/* to fork() */
 			    			
 			    if (fork_count <= MAX_RVNAMED_CHILDREN) {
+			        /*
+			         * If we can still fork(), we add the data
+			         * to the cache array, but we don't update
+			         * the indexes until after the fork()
+			         * succeeds.  If the fork() fails, we'll
+			         * just reuse this slot for the next query.
+			         *
+			         * This is so that if the fork() fails due
+			         * to a temporary condition, rvnamed won't
+			         * think it's RESOLVING while there isn't
+			         * any actual child doing the resolution
+			         * before the entry expires.
+			         *
+			         * However, we'll still tell IPTraf that the
+			         * address is RESOLVING.
+			         *
+			         */
+			        hostlist[hostindex].addr =
+				    rvnpacket.saddr.s_addr;
+			        hostlist[hostindex].ready = RESOLVING;
+
 			        fr = fork();
 			    } else {
 			        writervnlog(logfile, "Maximum child process limit reached");
@@ -389,6 +399,8 @@ int main(void)
 			    switch (fr) {
 			    case 0:			/* spawned child */
 				fclose(logfile);	/* no logging in child */
+				close(ifd);             /* no comm with client */
+				pid = getpid();
 				process_rvn_packet(&rvnpacket);
 				exit(0);
 			    case -1:
@@ -399,11 +411,21 @@ int main(void)
 				if (fork_count > max_fork_count)
 				    max_fork_count = fork_count;
 				
-				if (fork_count % 5 == 0) {
-				    sprintf(logmsg, "%d child processes", fork_count);
-				    writervnlog(logfile, logmsg);
-				}
-				    
+				/*
+				 * Increase cache indexes only if fork()
+				 * succeeded, otherwise the previously
+				 * allocated slots will be used for the
+				 * next query.
+				 */
+				     
+			        hostindex++;
+
+			        if (hostindex == NUM_CACHE_ENTRIES)
+				    hostindex = 0;
+
+			        if (lastfree < NUM_CACHE_ENTRIES)
+				    lastfree++;
+
 				fork_count++;
 				break;
 			    }
