@@ -58,10 +58,12 @@ details.
 #define SCROLLUP 0
 #define SCROLLDOWN 1
 
+#define LEFT 0
+#define RIGHT 1
+
 extern int exitloop;
 extern int daemonized;
 
-extern void show_sort_statwin(WINDOW **win, PANEL **panel);
 extern void writeutslog(struct portlistent *list, unsigned long nsecs,
 			FILE * logfile);
 
@@ -285,19 +287,9 @@ void printport(struct portlist *list, unsigned int protocol,
 }
 
 /*
- * Sorting for TCP/UDP service monitor, contributed by Gal Laszlo.
- * Revised, GPJ <riker@seul.org> 10/4/2000.
- * Contributor's note:
- *      by LG - Gal Laszlo ( SlowTCP@hotmail.com ) 2000.07.18. 
- *
- * Author's note: accented characters removed for better terminal
- * compatibility; comments in standard C style.
+ * Swap two port list entries.  p1 must be previous to p2.
  */
-
-/*
- * Swap two portlist nodes
- */
-
+ 
 void swapportents(struct portlist *list,
 		  struct portlistent *p1, struct portlistent *p2)
 {
@@ -339,15 +331,154 @@ void swapportents(struct portlist *list,
     p1->next_entry = p2nextsaved;
 }
 
+/*
+ * Retrieve the appropriate sort criterion based on keystroke.
+ */
+unsigned long long qp_getkey(struct portlistent *entry, int ch)
+{
+    unsigned long long result = 0;
+    
+    switch(ch) {
+    case 'R':
+        result = entry->port;
+        break;
+    case 'B':
+        result = entry->bcount;
+        break;
+    case 'O':
+        result = entry->ibcount;
+        break;
+    case 'M':
+        result = entry->obcount;
+        break;
+    case 'P':
+        result = entry->count;
+        break;
+    case 'T':
+        result = entry->icount;
+        break;
+    case 'F':
+        result = entry->ocount;
+        break;
+    }
+    
+    return result;
+}
+
+/*
+ * Compare the sort criterion with the pivot value.  Receives a parameter
+ * specifying whether the criterion is left or right of the pivot value.
+ *
+ * If criterion is the port number: return true if criterion is less than or
+ *     equal to the pivot when the SIDE is left.  If SIDE is right, return
+ *     true if the value is greater than the pivot.  This results in an
+ *     ascending sort.
+ *
+ * If the criterion is a count: return true when the criterion is greater than
+ *     or equal to the pivot when the SIDE is left, otherwise, when SIDE is
+ *     right, return true if the value is less than the pivot.  This results
+ *     in a descending sort. 
+ */
+ 
+int qp_compare(struct portlistent *entry, unsigned long long pv, int ch,
+                    int side)
+{
+    int result = 0;
+    unsigned long long value;
+    
+    value = qp_getkey(entry, ch);
+    
+    if (ch == 'R') {
+        if (side == LEFT)
+            result = (value <= pv);
+        else
+            result = (value > pv);
+    } else {
+        if (side == LEFT)
+            result = (value >= pv);
+        else
+            result = (value < pv);
+    }
+    
+    return result;
+}
+
+/*
+ * Partition port list such that a pivot is selected, and that all values
+ * left of the pivot are less (or greater) than or equal to the pivot,
+ * and that all values right of the pivot are greater (or less) than
+ * the pivot.
+ */
+struct portlistent *qp_partition(struct portlist *table,
+                                 struct portlistent **low,
+                                 struct portlistent **high, int ch)
+{
+    struct portlistent *pivot = *low;
+    
+    struct portlistent *left = *low;
+    struct portlistent *right = *high;
+    struct portlistent *ptmp;
+    
+    unsigned long long pivot_value;
+    
+    pivot_value = qp_getkey(pivot, ch);
+    
+    while (left->idx < right->idx) {
+        while ((qp_compare(left, pivot_value, ch, LEFT))
+               && (left->next_entry != NULL))
+            left = left->next_entry;
+            
+        while (qp_compare(right, pivot_value, ch, RIGHT))
+            right = right->prev_entry;
+            
+        if (left->idx < right->idx) {
+            swapportents(table, left, right);
+            if (*low == left)
+                *low = right;
+                
+            if (*high == right)
+                *high = left;
+                
+	    ptmp = left;
+	    left = right;
+	    right = ptmp;
+	}
+    }
+    swapportents(table, pivot, right);	    
+    
+    if (*low == pivot)
+        *low = right;
+                
+    if (*high == right)
+        *high = pivot;
+            
+    return pivot;
+}
+
+/*
+ * Quicksort for the port list.
+ */
+void quicksort_port_entries(struct portlist *table,
+                            struct portlistent *low,
+                            struct portlistent *high, int ch)
+{
+    struct portlistent *pivot;
+    
+    if ((high == NULL) || (low == NULL))
+        return;
+        
+    if (high->idx > low->idx) {
+        pivot = qp_partition(table, &low, &high, ch);
+        
+        quicksort_port_entries(table, low, pivot->prev_entry, ch);
+        quicksort_port_entries(table, pivot->next_entry, high, ch);
+    }
+}
+
 void sortportents(struct portlist *list, int *idx, int command)
 {
-    struct portlistent *ptemp1, *ptemp2;
-    struct portlistent *ptemp2next;
-
-    unsigned int long long val1 = 0, val2 = 0;
+    struct portlistent *ptemp1;
     unsigned int idxtmp;
-    WINDOW *win;
-    PANEL *panel;
     
     if (!list->head)
         return;
@@ -359,64 +490,7 @@ void sortportents(struct portlist *list, int *idx, int command)
        (command != 'F'))
         return;
         
-    ptemp1 = list->head;
-
-    show_sort_statwin(&win, &panel);
-    while (ptemp1 != NULL) {
-	ptemp2 = ptemp1->next_entry;
-
-        mvwprintw(win, 2, 13, "%u%%", ptemp1->idx * 100 / list->tail->idx);
-        update_panels();
-        doupdate();
-        
-	while (ptemp2 != NULL) {
-	    ptemp2next = ptemp2->next_entry;
-
-	    switch (command) {
-	    case 'R':		/* order by port number */
-		val2 = ptemp1->port;
-		val1 = ptemp2->port;
-		break;
-	    case 'B':		/* order by bytes */
-		val1 = ptemp1->bcount;
-		val2 = ptemp2->bcount;
-		break;
-	    case 'O':		/* order by bytesTo */
-		val1 = ptemp1->ibcount;
-		val2 = ptemp2->ibcount;
-		break;
-	    case 'M':		/* order by bytesFrom */
-		val1 = ptemp1->obcount;
-		val2 = ptemp2->obcount;
-		break;
-	    case 'P':		/* order by Pkts */
-		val1 = ptemp1->count;
-		val2 = ptemp2->count;
-		break;
-	    case 'T':		/* order by PktsTo */
-		val1 = ptemp1->icount;
-		val2 = ptemp2->icount;
-		break;
-	    case 'F':		/* order by PktsFrom */
-		val1 = ptemp1->ocount;
-		val2 = ptemp2->ocount;
-		break;
-	    }
-
-	    if (val1 < val2) {	/* descending sort */
-		swapportents(list, ptemp1, ptemp2);
-		ptemp1 = ptemp2;
-	    }
-	    ptemp2 = ptemp2next;
-	}
-
-	ptemp1 = ptemp1->next_entry;
-    }				/* end of the sorting */
-    
-    del_panel(panel);
-    delwin(win);
-    update_panels();
-    doupdate();
+    quicksort_port_entries(list, list->head, list->tail, command);
     
     ptemp1 = list->firstvisible = list->head;
     *idx = 1;
@@ -429,13 +503,6 @@ void sortportents(struct portlist *list, int *idx, int command)
     }
 }
 
-/* 
- * Contributor's note: 
- *  	end of "by LG"
- */
-
-
-
 void scrollservwin(struct portlist *table, int direction, int *idx)
 {
     char sp_buf[10];
@@ -581,7 +648,6 @@ void servmon(char *ifname, struct porttab *ports,
 	return;
     }
 
-
     open_socket(&fd);
 
     if (fd < 0) {
@@ -1102,14 +1168,14 @@ void selectport(struct porttab **table, struct porttab **node,
 	errbox("No custom ports", ANYKEY_MSG, &resp);
 	return;
     }
-    bw = newwin(20, 24, (LINES - 20) / 2 - 2, (COLS - 24) / 2 + 10);
+    bw = newwin(20, 24, (LINES - 20) / 2, (COLS - 24) / 2 - 5);
     bp = new_panel(bw);
     wattrset(bw, BOXATTR);
     box(bw, ACS_VLINE, ACS_HLINE);
     wmove(bw, 0, 1);
     wprintw(bw, " Select port entry ");
 
-    win = newwin(18, 22, (LINES - 18) / 2 - 2, (COLS - 22) / 2 + 10);
+    win = newwin(18, 22, (LINES - 18) / 2, (COLS - 22) / 2 - 5);
     panel = new_panel(win);
     stdwinset(win);
     wattrset(win, STDATTR);
