@@ -119,7 +119,9 @@ void init_tcp_table(struct tcptable *table)
     stdwinset(table->tcpscreen);
     print_tcp_num_entries(table);
 
-    /* Initialize hash table with dummy header cells */
+    /* 
+     * Initialize hash table to nulls
+     */
 
     for (i = 0; i <= ENTRIES_IN_HASH_TABLE - 1; i++) {
 	table->hash_table[i] = NULL;
@@ -162,10 +164,13 @@ int add_tcp_hash_entry(struct tcptable *table, struct tcptableent *entry)
     if (table->hash_table[hp] == NULL) {
 	ptmp->prev_entry = NULL;
 	table->hash_table[hp] = ptmp;
+        ptmp->index = 1;
     }
+
     if (table->hash_tails[hp] != NULL) {
 	table->hash_tails[hp]->next_entry = ptmp;
 	ptmp->prev_entry = table->hash_tails[hp];
+        ptmp->index = ptmp->prev_entry->index + 1;
     }
     table->hash_tails[hp] = ptmp;
     ptmp->next_entry = NULL;
@@ -211,7 +216,7 @@ struct tcptableent *addentry(struct tcptable *table,
 			     unsigned long int daddr,
 			     unsigned int sport, unsigned int dport,
 			     int protocol,  
-			     char *ifname, int rev_lookup,
+			     char *ifname, int *rev_lookup,
 			     int rvnfd, int servnames, int *nomem)
 {
     struct tcptableent *new_entry;
@@ -494,7 +499,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 		 struct tcphdr *transpacket, char *packet, int linkproto,
 		 unsigned long packetlength,
 		 unsigned int bcount, unsigned int fragofs, int logging,
-		 int mac, int revlook, int rvnfd, FILE * logfile, int *nomem)
+		 int mac, int *revlook, int rvnfd, FILE * logfile, int *nomem)
 {
     char msgstring[MSGSTRING_MAX];
     char newmacaddr[15];
@@ -609,6 +614,10 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	 *
 	 * Q: Could such a situation really happen in practice?  I managed to
 	 * do it but under *really* ridiculous circumstances.
+	 *
+	 * A: (as of version 2.5.0, June 2001): Yes this DOES happen in
+	 * practice.  Unidirectional satellite feeds can send data straight
+	 * to a remote network using you as your upstream.
 	 */
 
 	if (tableentry->oth_connection->pcount == 0)
@@ -658,7 +667,8 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 	time(NULL);
     /*
      * Shall we add this entry to the closed entry list?  If both
-     * directions have their state indicators set to 2, that's it.
+     * directions have their state indicators set to 2, or one direction
+     * is set to 2, and the other 1, that's it.
      */
 
     if ((!tableentry->inclosed) &&
@@ -895,7 +905,6 @@ void destroytcptable(struct tcptable *table)
      * Destroy list of closed entries
      */
 
-
     destroy_closed_entries(table);
 
     /*
