@@ -116,7 +116,7 @@ struct tcptableent *addentry(struct tcptable *table,
 			     unsigned long int daddr,
 			     unsigned int sport, unsigned int dport,
 			     int protocol,
-			     char *ifname, int rev_lookup, int rvnamedon,
+			     char *ifname, int *rev_lookup, int rvnamedon,
 			     int servnames,
 			     int *nomem);
 
@@ -130,7 +130,7 @@ void updateentry(struct tcptable *table, struct tcptableent *tableentry,
 		 struct tcphdr *transpacket, char *packet, int linkproto,
 		 unsigned long packetlength,
 		 unsigned int bcount, unsigned int fragofs, int logging,
-		 int mac, int revlook, int rvnfd, FILE * logfile, int *nomem);
+		 int mac, int *revlook, int rvnfd, FILE * logfile, int *nomem);
 
 void addtoclosedlist(struct tcptable *table, struct tcptableent *tableentry,
 		     int *nomem);
